import sys
from thales_remote.connection import ThalesRemoteConnection
from thales_remote.script_wrapper import ThalesRemoteScriptWrapper
from thales_remote.file_interface import ThalesFileInterface

from zahner_analysis.file_import.isc_import import IscImport

import matplotlib.pyplot as plt
import numpy as np
from matplotlib.ticker import EngFormatter


if __name__ == "__main__":
    zenniumConnection = ThalesRemoteConnection()
    zenniumConnection.connectToTerm("localhost", "ScriptRemote")

    zahnerZennium = ThalesRemoteScriptWrapper(zenniumConnection)
    zahnerZennium.forceThalesIntoRemoteScript()

    zahnerZennium.calibrateOffsets()

    fileInterface = ThalesFileInterface("localhost")
    fileInterface.disableSaveReceivedFilesToDisk()
    fileInterface.enableKeepReceivedFilesInObject()
    fileInterface.enableAutomaticFileExchange(fileExtensions="*.isc")

    zahnerZennium.setCVStartPotential(0)
    zahnerZennium.setCVUpperReversingPotential(0.2)
    zahnerZennium.setCVLowerReversingPotential(-0.2)
    zahnerZennium.setCVEndPotential(0)

    zahnerZennium.setCVStartHoldTime(2)
    zahnerZennium.setCVEndHoldTime(2)

    zahnerZennium.setCVCycles(1.5)
    zahnerZennium.setCVSamplesPerCycle(400)

    zahnerZennium.setCVMaximumCurrent(0.0002)
    zahnerZennium.setCVMinimumCurrent(-0.0002)

    zahnerZennium.setCVOhmicDrop(0)

    zahnerZennium.disableCVAutoRestartAtCurrentOverflow()
    zahnerZennium.disableCVAutoRestartAtCurrentUnderflow()
    zahnerZennium.disableCVAnalogFunctionGenerator()

    zahnerZennium.setCVNaming("individual")
    zahnerZennium.setCVOutputPath(r"C:\THALES\temp\cv")

    scanRatesForMeasurement = [0.5, 1, 2]

    for scanRate in scanRatesForMeasurement:
        zahnerZennium.setCVOutputFileName("cv_{:d}mVs".format(int(scanRate * 1000)))
        zahnerZennium.setCVScanRate(scanRate)

        zahnerZennium.checkCVSetup()

        zahnerZennium.measureCV()

        """
        Determine the maximum current for the next measurement
        from the maximum current of the last CV measurement.
        """
        latestMeasurement = IscImport(fileInterface.getLatestReceivedFile().binaryData)
        maximumCurrent = max(abs(latestMeasurement.getCurrentArray()))

        zahnerZennium.setCVMaximumCurrent(maximumCurrent * 3)
        zahnerZennium.setCVMinimumCurrent(maximumCurrent * -3)

    zenniumConnection.disconnectFromTerm()
    fileInterface.close()

    iscFileFromDisc = IscImport(r"C:\THALES\temp\cv\cv_1000mVs.isc")

    iscFiles = [IscImport(file.binaryData) for file in fileInterface.getReceivedFiles()]

    for iscFile in iscFiles:
        print(
            f"{iscFile.getScanRate()} V/s\tmeasurement finished at {iscFile.getMeasurementEndDateTime()}"
        )

    figCV, (axis) = plt.subplots(1, 1)
    figCV.suptitle("Cyclic Voltammetry at different scan rates")

    for iscFile in iscFiles:
        axis.plot(
            iscFile.getVoltageArray(),
            iscFile.getCurrentArray(),
            label=f"{iscFile.getScanRate()} $\\frac{{V}}{{s}}$",
        )

    axis.grid(which="both")
    axis.xaxis.set_major_formatter(EngFormatter(unit="$V$"))
    axis.yaxis.set_major_formatter(EngFormatter(unit="$A$"))
    axis.set_xlabel(r"$Voltage$")
    axis.set_ylabel(r"$Current$")
    axis.legend()

    figCV.set_size_inches(18, 18)
    plt.show()
    figCV.savefig("CV.svg")



import sys
from thales_remote.connection import ThalesRemoteConnection
from thales_remote.script_wrapper import PotentiostatMode, ThalesRemoteScriptWrapper
import time
import threading

zenniumConnection = None
zahnerZennium = None
keepThreadRunning = True


def watchThreadFunction():
    global zenniumConnection
    global zahnerZennium
    global keepThreadRunning

    while keepThreadRunning:
        time.sleep(1)
        active = zahnerZennium.getTermIsActive()
        print("active state: " + str(active))
        if active:
            print("beat count: " + str(zahnerZennium.getWorkstationHeartBeat()))


if __name__ == "__main__":
    zenniumConnection = ThalesRemoteConnection()
    zenniumConnection.connectToTerm("localhost", "ScriptRemote")

    zahnerZennium = ThalesRemoteScriptWrapper(zenniumConnection)
    zahnerZennium.forceThalesIntoRemoteScript()

    zahnerZennium.calibrateOffsets()

    testThread = threading.Thread(target=watchThreadFunction)
    testThread.start()
    print("heartbeat thread started")

    zahnerZennium.setPotentiostatMode(PotentiostatMode.POTMODE_POTENTIOSTATIC)
    zahnerZennium.setAmplitude(10e-3)
    zahnerZennium.setPotential(0)
    zahnerZennium.setLowerFrequencyLimit(500)
    zahnerZennium.setStartFrequency(1000)
    zahnerZennium.setUpperFrequencyLimit(10000)
    zahnerZennium.setLowerNumberOfPeriods(5)
    zahnerZennium.setLowerStepsPerDecade(2)
    zahnerZennium.setUpperNumberOfPeriods(20)
    zahnerZennium.setUpperStepsPerDecade(20)
    zahnerZennium.setScanDirection("startToMax")
    zahnerZennium.setScanStrategy("single")

    zahnerZennium.enablePotentiostat()

    zahnerZennium.setFrequency(1)
    zahnerZennium.setAmplitude(10e-3)
    zahnerZennium.setNumberOfPeriods(3)

    print("measurement start")
    zahnerZennium.measureEIS()
    print("measurement end")

    zahnerZennium.disablePotentiostat()

    print("thread kill")
    keepThreadRunning = False
    testThread.join()
    print("thread killed")

    zenniumConnection.disconnectFromTerm()
    print("finish")



import sys
from thales_remote.connection import ThalesRemoteConnection
from thales_remote.script_wrapper import PotentiostatMode, ThalesRemoteScriptWrapper
from thales_remote.file_interface import ThalesFileInterface
from zahner_analysis.file_import.ism_import import IsmImport

import matplotlib.pyplot as plt
import numpy as np
from matplotlib.ticker import EngFormatter
import os
import shutil

if __name__ == "__main__":
    remoteIP = "192.168.2.66"

    zenniumConnection = ThalesRemoteConnection()
    zenniumConnection.connectToTerm(remoteIP, "ScriptRemote")

    zahnerZennium = ThalesRemoteScriptWrapper(zenniumConnection)
    zahnerZennium.forceThalesIntoRemoteScript()

    zahnerZennium.calibrateOffsets()

    fileInterface = ThalesFileInterface(remoteIP)

    zahnerZennium.setEISOutputPath(r"C:\THALES\temp")
    zahnerZennium.setEISNaming("individual")
    zahnerZennium.setEISOutputFileName("myeis")

    zahnerZennium.setPotentiostatMode(PotentiostatMode.POTMODE_POTENTIOSTATIC)
    zahnerZennium.setAmplitude(100e-3)
    zahnerZennium.setPotential(0)
    zahnerZennium.setLowerFrequencyLimit(10)
    zahnerZennium.setStartFrequency(1000)
    zahnerZennium.setUpperFrequencyLimit(10000)
    zahnerZennium.setLowerNumberOfPeriods(5)
    zahnerZennium.setLowerStepsPerDecade(5)
    zahnerZennium.setUpperNumberOfPeriods(20)
    zahnerZennium.setUpperStepsPerDecade(5)
    zahnerZennium.setScanDirection("startToMax")
    zahnerZennium.setScanStrategy("single")

    zahnerZennium.enablePotentiostat()

    zahnerZennium.measureEIS()

    zahnerZennium.disablePotentiostat()
    zahnerZennium.setAmplitude(0)

    file = fileInterface.acquireFile(r"C:\THALES\temp\myeis.ism")

    fileHandle = open(r"D:\myLocalDirectory\asdf.ism", "wb")
    fileHandle.write(file.binaryData)
    fileHandle.close()

    ismFile = IsmImport(file.binaryData)

    impedanceFrequencies = ismFile.getFrequencyArray()

    impedanceAbsolute = ismFile.getImpedanceArray()
    impedancePhase = ismFile.getPhaseArray()

    figBode, (impedanceAxis, phaseAxis) = plt.subplots(2, 1, sharex=True)
    figBode.suptitle("Bode")

    impedanceAxis.loglog(
        impedanceFrequencies, impedanceAbsolute, marker="+", markersize=5
    )
    impedanceAxis.xaxis.set_major_formatter(EngFormatter(unit="Hz"))
    impedanceAxis.yaxis.set_major_formatter(EngFormatter(unit="$\Omega$"))
    impedanceAxis.set_xlabel(r"$f$")
    impedanceAxis.set_ylabel(r"$|Z|$")
    impedanceAxis.grid(which="both")

    phaseAxis.semilogx(
        impedanceFrequencies,
        np.abs(impedancePhase * (360 / (2 * np.pi))),
        marker="+",
        markersize=5,
    )
    phaseAxis.xaxis.set_major_formatter(EngFormatter(unit="Hz"))
    phaseAxis.yaxis.set_major_formatter(EngFormatter(unit="$°$", sep=""))
    phaseAxis.set_xlabel(r"$f$")
    phaseAxis.set_ylabel(r"$|Phase|$")
    phaseAxis.grid(which="both")
    phaseAxis.set_ylim([0, 90])
    figBode.set_size_inches(18, 12)
    plt.show()

    localDirectory = r"D:\myLocalDirectory"

    # Delete the entire contents of the directory.
    for file in os.listdir(localDirectory):
        fileWithPath = os.path.join(localDirectory, file)
        try:
            if os.path.isfile(fileWithPath) or os.path.islink(fileWithPath):
                os.unlink(fileWithPath)
            elif os.path.isdir(fileWithPath):
                shutil.rmtree(fileWithPath)
        except:
            pass

    fileInterface.enableSaveReceivedFilesToDisk(path=localDirectory)
    fileInterface.enableKeepReceivedFilesInObject()
    fileInterface.enableAutomaticFileExchange()

    zahnerZennium.setEISNaming("counter")
    zahnerZennium.setEISCounter(13)
    zahnerZennium.setEISOutputFileName("spectra")

    zahnerZennium.setupPAD4(1, 1, 1)
    zahnerZennium.setupPAD4(1, 2, 1)
    zahnerZennium.enablePAD4()

    zahnerZennium.measureEIS()
    fileInterface.disableAutomaticFileExchange()

    zahnerZennium.measureEIS()

    fileInterface.enableAutomaticFileExchange()
    zahnerZennium.measureEIS()
    zahnerZennium.setAmplitude(0)

    for file in fileInterface.getReceivedFiles():
        ismFile = IsmImport(file.binaryData)
        print(
            f"{file.name} measurement finished at {ismFile.getMeasurementEndDateTime()}"
        )

    for file in os.listdir(localDirectory):
        print(file)

    zenniumConnection.disconnectFromTerm()
    fileInterface.close()



import sys
from thales_remote.connection import ThalesRemoteConnection
from thales_remote.script_wrapper import ThalesRemoteScriptWrapper

if __name__ == "__main__":
    zenniumConnection = ThalesRemoteConnection()
    zenniumConnection.connectToTerm("localhost", "ScriptRemote")

    zahnerZennium = ThalesRemoteScriptWrapper(zenniumConnection)
    zahnerZennium.forceThalesIntoRemoteScript()

    zahnerZennium.calibrateOffsets()
    zahnerZennium.setSequenceNaming("dateTime")
    zahnerZennium.setSequenceOutputPath(r"C:\THALES\temp\test1")
    zahnerZennium.setSequenceOutputFileName("batterysequence")

    zahnerZennium.selectSequence(0)

    zahnerZennium.runSequence()

    zahnerZennium.selectPotentiostat(1)

    zahnerZennium.setSequenceNaming("counter")
    zahnerZennium.setSequenceCounter(13)
    zahnerZennium.setSequenceOutputPath(r"C:\THALES\temp\test1")
    zahnerZennium.setSequenceOutputFileName("batterysequence")

    zahnerZennium.runSequenceFile(r"C:\Users\XXX\Desktop\myZahnerSequence.seq")

    zahnerZennium.selectPotentiostat(0)

    zenniumConnection.disconnectFromTerm()
    print("finish")
